/*
 * CompactDiskCollection.java
 */

package catalog;

import java.util.*;
import catalog.compactdisk.*;
import catalog.datastore.*;


/**
 * A class to represent a singleton CompactDisc Catalog abstraction.
 */
public class CompactDiskCollection
{
	/**
  	* The maximum product code;
    */
  protected static int m_maxProductCode;

	/**
	  * The "Collection" member.
  	*/
 	protected static Map m_collection;


	/**
	 * The singleton instance of a CompactDiskCollection.
   */
  private static CompactDiskCollection m_instance;


	 /**
	  * Constructor.
	  * <p><b>pre:</b> True.
	  * <p><b>post:</b>An object created with a new collection member
	  *                and initialised.
	  */

	private CompactDiskCollection()
	{
	  this.m_instance = null;
  	m_collection = new TreeMap();
  }

	/**
	 * Accessor to the singleton instance.
	 * <p><b>pre:</b> True.
	 * <p><b>post:</b>A reference to the singleton object returned.
	 */
  public static CompactDiskCollection getInstance()
	{
    if (m_instance == null)
		  m_instance = new CompactDiskCollection();

    return m_instance;
  }

  /**
   * Mutator for the max product code member.
	 * <p><b>pre:</b> True.
	 * <p><b>post:</b>The member is reset to the larger between itself and
	 *       the given argument.
	 */
  public void setMaxProductCode(int productCode)
	{
    m_maxProductCode = productCode;
  }

  /**
   * Accessor for the maxProductCode.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>MaxProductCode returned.
   */
  public int getMaxProductCode()
  {
	  return m_maxProductCode;
  }

  /**
   * Adds a compact disk object to the collection.
   * <p><b>pre:</b> This object does not already exist in the collection.
   *               (check with containsTitle method).
   * <p><b>post:</b>The CompactDisk object is added to the collection.
   */
  public void addCompactDisk(CompactDisk cd)
  {
  	try
	  {
			CompactDisk cdKey = new CompactDisk(cd.getArtist().toLowerCase(),
		                        						  cd.getTitle().toLowerCase());
      m_collection.put(cdKey,cd);
		}
    catch (Exception ex)
		{
      System.out.println(ex.getMessage());
    }
  }

  /**
   * Returns an iterator on the collection.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>An Iterator over the collection is returned.
   */
  public Iterator iterator()
  {
    return m_collection.values().iterator();
  }

  /**
   * Returns the collection.
   * <p><b>pre:</b> True.
   * <p><b>post:</b> The collection is returned.
   */
  public Collection getCollection()
  {
  	return m_collection.values();
  }

  /**
   * Returns a boolean indicating if the collection contains
   * a compact disk record for the given artist and title.
   * <p><b>pre:</b> True
   * <p><b>post:</b>True if the artist/title exists, false otherwise.
   */
  public boolean containsTitle(String artist,
                               String title)
  {
	  boolean CDfound = false;

	  CompactDisk givenCD = new CompactDisk(artist.toLowerCase(),
                                          title.toLowerCase());

		CDfound = m_collection.containsKey(givenCD);

		return CDfound;
  }

  /**
   * Returns the CompactDisk object that has the artist and
   * title given as parameters to the method.
   * <p><b>pre:</b> A compact disk entry exists for the given artist
   *       and title given (check with containsTitle method).
   * <p><b>post:</b>A reference to a CompactDisk object returned.
   */
  public CompactDisk obtainCompactDisk(String artist, String title)
  {
	  CompactDisk searchCD = new CompactDisk(artist.toLowerCase(),
                                           title.toLowerCase());
		CompactDisk foundCD = (CompactDisk)m_collection.get(searchCD);
    return foundCD;
  }

  /**
   * Update the CompactDisk object that has the artist and
   * title given as parameters to the method.
   * <p><b>pre:</b> A compact disk entry exists for the given artist
   *       and title given (check with containsTitle method).
   * <p><b>post:</b>The CD in the CD collection has been updated
   */
  public void updateCompactDisk(CompactDisk cd)
  {
    this.obtainCompactDisk(cd.getArtist(),
                           cd.getTitle()).setStockCount(cd.getStockCount());
  }

} // eof.

