/*
 * CompactDisk.java
 */

package catalog.compactdisk;

import java.io.*;
import java.lang.Comparable;


/**
 * A class representing the Compact Disk abstraction.
 */
public class CompactDisk
  implements Comparable
{
  /**
   * The product code of this compact disk (a unique id).
   */
  protected int m_productCode;

  /**
   * The artist who recorded this compact disk.
   */
  protected String m_artist;

  /**
   * The compact disk title.
   */
  protected String m_title;

  /**
   * The year of publication.
   */
  protected String m_year;

  /**
   * The cover image file name.
   */
  protected String m_cover;

  /**
   * The count of current stock.
   */
  protected int m_stockCount;

  /**
   * Constructs a <code>CompactDisk</code> object using the given
   * arguments.
   * <p><b>pre:</b> All arguments are valid.
   * <p><b>post:</b>An object created.
   */
  public CompactDisk(int productCode,
                     String artistName,
                     String compactDiskTitle,
                     String yearOfPublication,
                     String cover,
                     int stockCount)
  {
    this.m_productCode = productCode;
    this.m_artist = artistName;
    this.m_title = compactDiskTitle;
    this.m_year = yearOfPublication;
    this.m_cover = cover;
    this.m_stockCount = stockCount;
  }

  /**
   * Constructs a <code>CompactDisk</code> object using two
   * arguments.
   * <p><b>pre:</b> All arguments are valid.
   * <p><b>post:</b>An object created.
   */
  public CompactDisk(String artistName,
                     String compactDiskTitle)
  {
    this.m_artist = artistName;
    this.m_title = compactDiskTitle;
  }

  /**
   * Constructs an empty <code>CompactDisk</code> object
   * <p><b>pre:</b> True.
   * <p><b>post:</b>An empty object created.
   */
  public CompactDisk()
  {

  }

  /**
   * Compares this object to the given object using Artist/Title
   * comparison order. Character case of the strings is ignored.
   * <p><b>pre:</b> otherDisk is of type CompactDisk.
   * <p><b>post:</b>Comparison result returned. The result == 0 if identical,
   *                result > 0 or result < 0 otherwise, depending upon the
   *                character collating sequence.
   */
  public int compareTo(Object otherDisk) throws ClassCastException
  {
    try
    {
      String param_temp_title,
             param_temp_artist,
             this_temp_title,
             this_temp_artist;

      // get title and artist name from object in parameter
      param_temp_title = ((CompactDisk)otherDisk).getTitle().toLowerCase();
      param_temp_artist = ((CompactDisk)otherDisk).getArtist().toLowerCase();

      // get title and artist name of this object
      this_temp_title = m_title.toLowerCase();
      this_temp_artist = m_artist.toLowerCase();

      // compare CompactDisk
      if ((this_temp_artist.compareTo(param_temp_artist))==0)
      {
        if ((this_temp_title.compareTo(param_temp_title))==0)
          return 0;
        else if ((this_temp_title.compareTo(param_temp_title))>0)
          return 1;
        else
          return -1;
      }
      else if((this_temp_artist.compareTo(param_temp_artist))>0)
        return 1;
      else
        return -1;
    }
    catch(ClassCastException e)
    {
      throw new ClassCastException(e.toString());
    }
  }

  /**
   * Check whether this object is equals to the object using Artist/Title
   * comparison order. Character case of the strings is ignored.
   * <p><b>pre:</b> otherDisk is of type CompactDisk.
   * <p><b>post:</b>Comparison result returned. True if equals, else false
   */
  boolean equals(CompactDisk cd)
  {
    if (this.compareTo(cd)==0)
      return true;
    return false;
  }

  /**
   * Accessor to the productCode unique id member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The product code is returned.
   */
  public int getProductCode()
  {
    return m_productCode;
  }

  /**
   * Accessor to artist member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>Reference to the artist member returned.
   */
  public String getArtist()
  {
    return m_artist;
  }

  /**
   * Accessor to title member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>Reference to the title member returned.
   */
  public String getTitle()
  {
    return m_title;
  }

  /**
   * Accessor to year member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The year member returned.
   */
  public String getYear()
  {
    return m_year;
  }

  /**
   * Accessor to the cover image member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>Reference to the String containing the file name
   *       of the image file is returned.
   */
  public String getCover()
  {
    return m_cover;
  }

  /**
   * Accessor to the stock count member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The value of the member is returned.
   */
  public int getStockCount()
  {
    return m_stockCount;
  }

  /**
   * Mutator of the stock count member.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The value of the member is set to the new value.
   */
  public void setStockCount(int stockCount)
  {
    this.m_stockCount = stockCount;
  }
}

// eof.
