/*
 * DataStore.java
 */

package catalog.datastore;

import catalog.compactdisk.*;
import catalog.CompactDiskCollection;


/**
 * An interface to provide a wrapper for a data store.
  */
public interface DataStore{

  /**
   * Method to open the data store.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>The datastore is opened.
   */
  public void open()
    throws OpenDataStoreException;
  
  /**
   * Method to close the data store.
   * <p><b>pre:</b> The datastore is open.
   * <p><b>post:</b>The data store is closed.
   */
  public void close() 
    throws CloseDataStoreException;
  
  /**
   * Report the details of the connection.
   * <p><b>pre:</b> A connection has been established by a successful call
   *       to method open().
   * <p><b>post:</b>Details of the connection are displayed to standard output.
   */
  public void reportConnectionDetails();
  
  /**
   * Used to load all the CompactDisk records from the store.
   * <p><b>pre:</b>The datastore is open. 
   * <p><b>post:</b>All cd's are loaded from the datastore into the 
   *      Collection. The maximum product code of the Collection is set.
   */
  public CompactDiskCollection 
  loadCompactDiskCollection(CompactDiskCollection cdCollection)
    throws LoadDataStoreException;
  
  /**
   * Used to save a CompactDisk object to the data store.
   * <p><b>pre:</b> The datastore is open. The given cd parameter is valid.
   * <p><b>post:</b>The given cd object has been written to the datastore.
   */
  public void saveCompactDisk(CompactDisk cd)
    throws SaveDataStoreException;

  /**
   * Used to update a CompactDisk object in the data store.
   * <p><b>pre:</b> The datastore is open. The compact disk parameter is valid.
   * <p><b>post:</b>The stock count in the datastore is updated for the
   *       CompactDisk details given.
   */
  public void updateCompactDisk(CompactDisk cd)
    throws SaveDataStoreException;

} // eof.
