/*
 * AddGUI.java
 */

package catalog.guiutil;

import java.io.*;
import java.io.File;
import java.io.FileFilter;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.ImageIcon;
import catalog.datastore.*;
import catalog.compactdisk.*;
import catalog.CompactDiskCollection;


/**
 * A class that renders "Add new CD" GUI (graphical user interface) and handle
 * events related to GUI components in this class.
 */
public class AddGUI extends JFrame
  implements ActionListener
{
  // Instance variables for labels
  private JLabel artistLabel,
                 titleLabel,
                 yearLabel,
                 stockLabel,
                 imageFileLabel,
                 imageIconLabel;

  // Instance variables for panels
  private JPanel upperPanel,
                 middlePanel,
                 lowerPanel,
                 labelPanel,
                 fieldPanel,
                 innerLowerPanel1,
                 innerLowerPanel2;

  // Instance variables for text fields
  private JTextField artistTf,
                     titleTf,
                     yearTf,
                     stockTf;

  // Instance variables for button
  private JButton commitButton,
                  clearButton,
                  musicButton,
                  imageFileButton;

  // Instance variable for image icon
  private Icon coverImage;

  // Instance variable for file chooser
  private JFileChooser fileChooser;

  // Instance variables for color
  private Color white,
                navyBlue,
                black,
                deepNavyBlue,
                gray;

  // Instance variable for font
  private Font font;

  // Instance variable for file
  private File file;

  // Instance variable for music player
  MusicPlayer musicPlayer;

  // Instance variable for SearchGUI which instantiate this class
  private StartupGUI m_owner;

  // Text field height
  private int fieldHeight = 26;

  // Text field length
  private int fieldLenght = 200;


  /**
   * Constructs a <code>AddGUI</code> object using the given
   * arguments. GUI components has been rendered on screen
   * <p><b>pre:</b> Arguments are valid.
   * <p><b>post:</b>An object created and GUI components has been
   *                displayed on screen.
   */
  public AddGUI(StartupGUI owner)
  {
    super("Add a new CD");
    m_owner = owner;

    // set the dialog box to not resizable
    setResizable(false);

    // set location for this frame
    setLocation(287,80);

    // Create white color
    white = new Color(255,255,255);

    // Create navy blue color
    navyBlue = new Color(50,70,220);

    // Create black color
    deepNavyBlue = new Color(0,0,128);

    // Create black color
    black = new Color(0,0,0);

    // Create gray color
    gray = new Color(178,178,178);

    // Listen for close window event
    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        if(musicPlayer!=null)
        {
          musicPlayer.stop();
          musicPlayer.destroy();
        }
        AddGUI.this.dispose();
      }
    });


    // Cover button and panel
    imageFileButton = new JButton("            Cover           ");
    imageFileButton.setBackground(navyBlue);
    imageFileButton.setForeground(white);
    imageFileButton.setToolTipText("Select cover image for CD");
    imageFileButton.addActionListener(this);


    // Commit button
    commitButton = new JButton("Commit");
    commitButton.setForeground(white);
    commitButton.setBackground(navyBlue);
    commitButton.setToolTipText("Search CD");
    commitButton.addActionListener(this);


    // Clear button
    clearButton = new JButton(" Clear ");
    clearButton.setForeground(white);
    clearButton.setBackground(navyBlue);
    clearButton.setToolTipText("Clear fields");
    clearButton.addActionListener(this);


    // Play music button
    musicButton = new JButton("Play music");
    musicButton.setForeground(white);
    musicButton.setBackground(black);
    musicButton.setToolTipText("Play music");
    musicButton.addActionListener(this);


    // artist label
    artistLabel = new JLabel("Artist");
    artistLabel.setBackground(white);
    artistLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 12);
    artistLabel.setFont(font);


    // title label
    titleLabel = new JLabel("Title");
    titleLabel.setBackground(white);
    titleLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    titleLabel.setFont(font);


    // year label
    yearLabel = new JLabel("Year");
    yearLabel.setBackground(white);
    yearLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    yearLabel.setFont(font);


    // stock label
    stockLabel = new JLabel("Stock");
    stockLabel.setBackground(white);
    stockLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    stockLabel.setFont(font);


    // image file label and panel
    imageFileLabel = new JLabel("Image File");
    imageFileLabel.setBackground(white);
    imageFileLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    imageFileLabel.setFont(font);


    // artist textField
	  artistTf = new JTextField("",20);
	  artistTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // title textField
	  titleTf = new JTextField("",20);
	  titleTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // year textField
 	  yearTf = new JTextField("",20);
	  yearTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // stock textField
	  stockTf = new JTextField("",20);
	  stockTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // ImageIcon label
    imageIconLabel = new JLabel();
    imageIconLabel.setBackground(white);


    // Create rigid area for text fields and labels

    // Label panel
   	labelPanel = new JPanel();
   	labelPanel.setOpaque(false);
    labelPanel.setBackground(white);
   	labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
   	labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(artistLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(titleLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(yearLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,11)));
    labelPanel.add(stockLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(0,11)));
    labelPanel.add(imageFileLabel);

    // Field panel
    fieldPanel = new JPanel();
  	fieldPanel.setOpaque(false);
    fieldPanel.setBackground(white);
   	fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.Y_AXIS));
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(artistTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(titleTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(yearTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(stockTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(imageFileButton);

    // Upper panel for textfields and panels
    upperPanel = new JPanel();
    upperPanel.setBackground(white);
    upperPanel.setLayout(new GridLayout(1,2));
    upperPanel.add(labelPanel);
    upperPanel.add(fieldPanel);


    // Middle panel
    middlePanel = new JPanel();
    middlePanel.setBackground(white);
    middlePanel.add(imageIconLabel);

    
    // Lower panel
    lowerPanel = new JPanel();
    lowerPanel.setBackground(white);
    lowerPanel.setBorder(BorderFactory.createLineBorder(gray,2));
    lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.Y_AXIS));


    // Inner part of the lower panel for search button and clear button
    innerLowerPanel1 = new JPanel();
    innerLowerPanel1.setBackground(white);
  	innerLowerPanel1.setLayout(new BoxLayout(innerLowerPanel1, BoxLayout.X_AXIS));

    innerLowerPanel1.add(commitButton);
   	innerLowerPanel1.add(Box.createRigidArea(new Dimension(5,0)));
    innerLowerPanel1.add(clearButton);


    // Inner part of the lower panel for play music button
    innerLowerPanel2 = new JPanel();
    innerLowerPanel2.setBackground(white);
    innerLowerPanel2.setLayout(new BoxLayout(innerLowerPanel2, BoxLayout.X_AXIS));
    innerLowerPanel2.add(Box.createRigidArea(new Dimension(15,0)));
    innerLowerPanel2.add(musicButton);
    innerLowerPanel2.add(Box.createRigidArea(new Dimension(15,0)));


    // Add all panels into lower panel
    lowerPanel.add(Box.createRigidArea(new Dimension(0,20)));
    lowerPanel.add(innerLowerPanel1);
    lowerPanel.add(Box.createRigidArea(new Dimension(0,15)));
    lowerPanel.add(innerLowerPanel2);
    lowerPanel.add(Box.createRigidArea(new Dimension(10,10)));


    // Add all components into the frame container
    Container Cp = getContentPane();
    Cp.setBackground(white);
    Cp.setLayout(new GridLayout(3,1));
    Cp.add(upperPanel);
    Cp.add(middlePanel);
    Cp.add(lowerPanel);

    setSize(280,420);
    setVisible(true);
    validate();
  }


  /**
   * Listen for ActionEvent generated by component in this class and
   * perform specific functions such as open image file chooser, commit
   * transaction or clear all text fields.
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> openImgFile method will be invoke if imageFileButton
   *                 has been clicked.
   *                 User's input will be validate and AddCD method will
   *                 be invoke if commitButton has been clicked.
   *                 TextFields and imageIcon will be cleared if clearButton
   *                 has been clicked.
   */
	public void actionPerformed(ActionEvent e)
	{
    if(e.getSource()==imageFileButton)
    {
      openImgFile();
    }

    if(e.getSource()==commitButton)
    {
      // validate input for artistTf
      if (artistTf.getText().length()==0)
      {
        JOptionPane.showMessageDialog(null, "Please enter artist name",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // validate input for titleTf
      if (titleTf.getText().length()==0)
      {
        JOptionPane.showMessageDialog(null, "Please enter title name",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // validate input for yearTf
      if (yearTf.getText().length()!=4)
      {
        JOptionPane.showMessageDialog(null, "Please enter valid year value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // validate input for stockTf
      if ( (stockTf.getText().length()==0) ||
           (stockTf.getText().length()<0) ||
           (stockTf.getText().length()>5) )
      {
        JOptionPane.showMessageDialog(null, "Please enter valid stock value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // validate input for coverImage
      if (coverImage==null)
      {
        JOptionPane.showMessageDialog(null, "Please select cover image",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // check whether stock is type of integer
      try
      {
        new Integer(stockTf.getText()).intValue();
      }
      catch(NumberFormatException err)
      {
        JOptionPane.showMessageDialog(null, "Please enter valid stock value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // check whether year is type of integer
      try
      {
        new Integer(yearTf.getText()).intValue();
      }
      catch(NumberFormatException err)
      {
        JOptionPane.showMessageDialog(null, "Please enter valid year value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // Invoke addCD method in this class
      addCD();

      // Tell StartupGUI that that there is a change in cd collection
      m_owner.propertyChanged();
    }

    // If clear button has been clicked then clear all textfield
    // and clear image
    if(e.getSource()==clearButton)
    {
      artistTf.setText("");
      titleTf.setText("");
      yearTf.setText("");
      stockTf.setText("");
      coverImage = null;
      imageIconLabel.setIcon(coverImage);
    }

    // If music button has been clicked then create player and play music
    if(e.getSource()==musicButton)
    {
      // Create instance of musicPlayer
      musicPlayer = new MusicPlayer();

      // Start the music
      musicPlayer.start();

      // Add music player into lower panel
      lowerPanel.add(musicPlayer);

      // update lower panel
      lowerPanel.updateUI();
    }
	}


  /**
   * Open file chooser to select cover image for new cd
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Image chooser dialog box has been created, user can
   *                 select image file from image chooser and cover image
   *                 will be display on the AddGUI Frame box.
   */
	private void openImgFile()
	{
    // get the current work directory
		File f = new File (System.getProperty("user.dir"));

    // set the current working directory to the Cover images Directly
  	String currentDirectory = f + file.separator + "coverimages";

	  // Filechooser with the current working directory as the parameter
	  fileChooser = new JFileChooser(currentDirectory);

    // make visible only the GIF & JPG files
	  fileChooser.setFileFilter(new ImageFilter());

    // Add file preview accessories
    fileChooser.setAccessory(new ImagePreview(fileChooser));

    // can only select files no directories
  	fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

    // Set file to null if the file chooser is cancelled
		if(fileChooser.showOpenDialog(this)==JFileChooser.CANCEL_OPTION)
    {
			file = null;
    }
		else
		{
			file = fileChooser.getSelectedFile();

			try
      {
			  coverImage = new ImageIcon(file.toURL());
			}
      catch(MalformedURLException e)
      {
        System.err.println(e);
      }
      // Display selected image on screen
			imageIconLabel.setIcon(coverImage);
		}
	}


  /**
   * Read input from GUI components and create new CD and add it into
   * internal data structure and datastore
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Data input has been read from GUI components.
   *                 New CD has been created and both the internal
   *                 data structure and datastore has been updated.
   */
  private void addCD()
  {
    try
    {
      // Get singleton instance of CD collection
      CompactDiskCollection cds = CompactDiskCollection.getInstance();

      // Create an object of new compact disk
      CompactDisk newCD = new CompactDisk
      (
        cds.getMaxProductCode(),
        artistTf.getText(),
        titleTf.getText(),
        yearTf.getText(),
	      "coverimages" + file.separator + fileChooser.getName(file),
        new Integer(stockTf.getText()).intValue()
      );

      cds.setMaxProductCode(cds.getMaxProductCode() + 1);

      // Update the data in datastore (database/text file)
      DataStore data = DataStoreFactory.makeDataStore(
                           DataStoreOptionGUI.getDataStoreType());

      // Open datastore connection
      data.open();

      // Save data to datastore
      data.saveCompactDisk(newCD);

      // Close datastore connection
      data.close();

      // Update the disk collection in computer memory
      cds.addCompactDisk(newCD);
    }
    catch(NumberFormatException err)
    {
      JOptionPane.showMessageDialog(null, "Invalid integer in stock field",
                                    "Error", JOptionPane.ERROR_MESSAGE);
    }
    catch(OpenDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch(SaveDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch(CloseDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }
} // eof.


