/*
 * DataStoreOptionGUI.java
 */

package catalog.guiutil;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import catalog.datastore.*;
import catalog.CompactDiskCollection;
import catalog.compactdisk.CompactDisk;


/**
 * A class that renders modal dialog box that allows the user to choose
 * the type of datastore
 */
public class DataStoreOptionGUI extends JDialog implements ActionListener
{
  // Instance variables for panel
  private JPanel checkBoxPanel,
                 buttonPanel,
                 labelPanel;

  // Instance variables for checkboxes
  private JCheckBox fileBox,
                    dbBox;

  // Instance variables for checkbox group
  private ButtonGroup CGroup;

  // Instance variable for color
  private Color black;

  // Instance variables for button
  private JButton okButton;

  // Instance variable for label
  private JLabel label;

  // Instance variable for string
  private static String m_datatype;


  public DataStoreOptionGUI(StartupGUI owner)
  {
    // set title for frame
    super(owner, "Data store option", true);

    // set location for this frame
    setLocation(180,180);

    // Initialise black color
    black = new Color(0,0,0);

    // Listen for close window event
    addWindowListener(new WindowAdapter()
    {
       public void windowClosing(WindowEvent e)
       {
         loadData();
         DataStoreOptionGUI.this.dispose();
       }
    });


    // Initialize label
    label = new JLabel("Please Select type of datastore :");


    // Initialize label panel
    JPanel labelPanel = new JPanel();
    labelPanel.add(label, BorderLayout.WEST);
    label.setForeground(black);


    // Initialize checkboxes
    CGroup = new ButtonGroup();

    dbBox = new JCheckBox("Database ", true);
    dbBox.addActionListener(this);
    dbBox.setToolTipText("Database based data store");

    fileBox = new JCheckBox("Text file", false);
    fileBox.setToolTipText("Text file based data store");
    fileBox.addActionListener(this);

    m_datatype = "Database";
    CGroup.add(dbBox);
    CGroup.add(fileBox);


    // Initialize checkbox panel
    checkBoxPanel = new JPanel();
    checkBoxPanel.add(dbBox);
    checkBoxPanel.add(fileBox);


    // Initialize button
    okButton = new JButton("      Ok      ");
    okButton.setToolTipText("Proceed with selected datastore");
    okButton.setMnemonic('O');
    okButton.addActionListener(this);


    // Initialize button panel
    buttonPanel = new JPanel();
    buttonPanel.add(okButton);


    // Add all components into the frame container
    Container Cp = getContentPane();
    Cp.add(labelPanel, BorderLayout.NORTH);
    Cp.add(checkBoxPanel, BorderLayout.CENTER);
    Cp.add(buttonPanel, BorderLayout.SOUTH);

    setSize(230,120);
    setVisible(true);
    validate();
  }

	/**
	 * Accessor to datastore type choosed by user.
	 * <p><b>pre:</b> True.
	 * <p><b>post:</b>A reference to the of datastore type returned.
	 */
  public static String getDataStoreType()
  {
    return m_datatype;
  }

  /**
   * Listen for ActionEvent generated by component in this class
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> DataStore type will be defined as database if database
   *                 data store radio button has been choosen.
   *                 DataStore type will be defined as text file if file
   *                 data store radio button has been choosen.
   *                 loadData method will be invoked if the ok button has
   *                 been clicked
   */
	public void actionPerformed(ActionEvent e)
	{
    if(e.getSource()==dbBox)
      m_datatype = "Database";

		if(e.getSource()==fileBox)
      m_datatype = "File";

    if(e.getSource()==okButton)
    {
      loadData();
      this.dispose();
    }
	}

	/**
	 * Load data from data source
	 * <p><b>pre:</b> DataStore source has been defined.
	 * <p><b>post:</b> Data has been load into internal data structure
	 */
  private void loadData()
  {
    // Get instance of data store
    DataStore data =
      (DataStore)DataStoreFactory.makeDataStore(DataStoreOptionGUI.getDataStoreType());

    try
    {
      // Open data source
      data.open();

      // Load data
      CompactDiskCollection cds = CompactDiskCollection.getInstance();
      cds = data.loadCompactDiskCollection(cds);

      // Close data connection
      data.close();
    }
    catch(OpenDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                       JOptionPane.ERROR_MESSAGE);
    }
    catch(LoadDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                       JOptionPane.ERROR_MESSAGE);
    }
    catch(CloseDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                       JOptionPane.ERROR_MESSAGE);
    }
  }

} // eof.
