/*
 * ImagePreview.java
 */

package catalog.guiutil;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.File;


/**
 * A class which is a component that provide image preview for file chooser
 * dialog
 */
public class ImagePreview extends JComponent
                          implements PropertyChangeListener
{
  // Instance variable for image icon
  ImageIcon thumbnail = null;

  // Instance variable for file
  File file = null;

  /**
   * Constructs a <code>ImagePreview</code> object using the given
   * arguments.
   * <p><b>pre:</b> All arguments are valid.
   * <p><b>post:</b>An object created.
   */
  public ImagePreview(JFileChooser fc)
  {
    setPreferredSize(new Dimension(100, 50));
    fc.addPropertyChangeListener(this);
  }

  /**
   * Load image into image icon
   * <p><b>pre:</b> file object has been created.
   * <p><b>post:</b> Image has been loaded into the image icon.
   */
  public void loadImage()
  {
    if(file == null)
      return;

    ImageIcon tmpIcon = new ImageIcon(file.getPath());

    if (tmpIcon.getIconWidth() > 90)
    {
      thumbnail = new ImageIcon(tmpIcon.getImage().getScaledInstance(
                                90, -1, Image.SCALE_DEFAULT));
    }
    else
    {
      thumbnail = tmpIcon;
    }
  }

  /**
   * Listen for property change
   * <p><b>pre:</b> file object has been created.
   * <p><b>post:</b> Image has been loaded into the image icon.
   */
  public void propertyChange(PropertyChangeEvent e)
  {
    String prop = e.getPropertyName();
    if (prop.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY))
    {
      file = (File) e.getNewValue();
      if (isShowing())
      {
        loadImage();
        repaint();
      }
    }
  }

  /**
   * Paint the image icon on screen
   * <p><b>pre:</b> True.
   * <p><b>post:</b> loadImage method has been invoked and image has been
   *                 rendered on screen.
   */
  public void paintComponent(Graphics g)
  {
    if (thumbnail == null)
    {
      loadImage();
    }

    if (thumbnail != null)
    {
      int x = getWidth()/2 - thumbnail.getIconWidth()/2;
      int y = getHeight()/2 - thumbnail.getIconHeight()/2;

      if (y < 0)
      {
        y = 0;
      }

      if (x < 5)
      {
        x = 5;
      }
      thumbnail.paintIcon(this, g, x, y);
    }
  }
} // eof.
