/*
 * ListGui.java
 */

package catalog.guiutil;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;
import catalog.datastore.*;
import catalog.compactdisk.*;
import catalog.CompactDiskCollection;
import javax.swing.table.TableColumn;
import javax.swing.table.JTableHeader;


/**
 * A singleton class that renders a JTable to display all CDs available.
 */

public class ListGUI extends JFrame implements TableModelListener
{
  // Singleton instance of ListGUI
  private static ListGUI m_instance;

  // Instace variable for panel
  private JPanel buttonPanel;

  // Instance variable for table
  private JTable table;

  // Instace variable for button
  private JButton closeButton;

  // Instance variables for color
  private Color white,
                navyBlue;

  // Instance variable for JScrollPane
  private JScrollPane scroll;

  // Column name
  private String[] columnNames = {"Artist", "Title", "Year", "Stock"};


  /**
   * Constructs a <code>ListGUI</code> object and renders the GUI components
   * on screen.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>An object created and GUI components has been rendered on
   *                screen.
   */
  public ListGUI()
  {
    super("BOF Compact Disk Inventory");

    // Create white color
    white = new Color(255,255,255);

    // Create navy blue color
    navyBlue = new Color(50,70,220);

    // Set size of frame
    setSize(450,305);

    // set location for this frame
    setLocation(5,300);

    // Listen for close window event
    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        m_instance = null;
        ListGUI.this.dispose();
      }
    });


    // Initialize button and panel
    closeButton = new JButton("  Close  ");
    closeButton.setForeground(white);
    closeButton.setBackground(navyBlue);
    buttonPanel = new JPanel();
    buttonPanel.setBackground(white);
   	buttonPanel.add(closeButton, BorderLayout.SOUTH);
    closeButton.addActionListener(
    new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
		  {
        m_instance = null;
        ListGUI.this.dispose();
      }
    });

    // Initialize table
    table = new JTable(this.getSortedRows(), this.getColumns());
    table.setBackground(white);


    // Set size of particular column
    TableColumn column = null;
    column = table.getColumnModel().getColumn(0);
    column.setPreferredWidth(170);
    column = table.getColumnModel().getColumn(1);
    column.setPreferredWidth(230);


    // Create Scroll pane for table
    scroll = new JScrollPane(table);
    table.setPreferredScrollableViewportSize(new Dimension(425,221));


    // Add all components into the frame container
    Container Cp = getContentPane();
    Cp.setBackground(white);
    Cp.add(scroll, BorderLayout.NORTH);
    Cp.add(buttonPanel, BorderLayout.CENTER);

    setVisible(true);
    validate();
  }

  public static ListGUI getInstance()
  {
    if (m_instance == null)
      m_instance = new ListGUI();
    return m_instance;
  }

	public void tableChanged(TableModelEvent e)
	{
	  if(e.getSource() == table)
	  {
      update();
    }
	}

  /**
   * Update the view of table
   * <p><b>pre:</b> A table object has been created.
   * <p><b>post:</b> Table has been updated with the internal data structure.
   */
  public void update()
  {
    // unload outdated components
    Container Cp = getContentPane();
    Cp.removeAll();

    // Initialize button and panel
    closeButton = new JButton("  Close  ");
    closeButton.setForeground(white);
    closeButton.setBackground(navyBlue);
    buttonPanel = new JPanel();
    buttonPanel.setBackground(white);
   	buttonPanel.add(closeButton, BorderLayout.SOUTH);

    // Initialize table
    table = new JTable(this.getSortedRows(), this.getColumns());
    table.setBackground(white);


    // Set size of particular column
    TableColumn column = null;
    column = table.getColumnModel().getColumn(0);
    column.setPreferredWidth(170);
    column = table.getColumnModel().getColumn(1);
    column.setPreferredWidth(230);


    // Create Scroll pane for table
    scroll = new JScrollPane(table);
    table.setPreferredScrollableViewportSize(new Dimension(425,221));


    // Add all components into the frame container

    Cp.setBackground(white);
    Cp.add(scroll, BorderLayout.NORTH);
    Cp.add(buttonPanel, BorderLayout.CENTER);

    setVisible(true);
    validate();
  }

  /**
   *  Returns column for the table.
   * <p><b>pre:</b> columnNames has been created.
   * <p><b>post:</b> columns has been returned.
   */
	private Vector getColumns()
  {
	  Vector columns = new Vector();
	  for (int i=0; i<4; i++)
      columns.addElement(columnNames[i]);

    return columns;
	}

  /**
   * Return the rows for the table
   * <p><b>pre:</b> CompactDiskCollection has been created.
   * <p><b>post:</b> rows has been returned.
   */
	private Vector getSortedRows()
	{
    Vector rows = new Vector();
    CompactDiskCollection cds = CompactDiskCollection.getInstance();
    Iterator it = cds.iterator();

	  while(it.hasNext())
	  {
	    CompactDisk cd = (CompactDisk)it.next();
	    Vector elementsInRow = new Vector();
	    elementsInRow.addElement(cd.getArtist());
	    elementsInRow.addElement(cd.getTitle());
	    elementsInRow.addElement(cd.getYear());
	    elementsInRow.addElement(new Integer (cd.getStockCount()).toString());

	    rows.addElement(elementsInRow);
    }

	  return rows;
	}
} // eof.
