/*
 * MusicFilter.java
 */

package catalog.guiutil;

import java.io.File;
import javax.swing.*;
import javax.swing.filechooser.*;


/**
 * A class that filters files other than image files with .mp3 extension.
 */
public class MusicFilter extends FileFilter
{
  /**
   * Method that return true for all directories and all mp3 files.
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> Return true if selected file object is directory, gif,
   *                 jpg or jpeg files.
   */
  public boolean accept(File f)
  {
    if (f.isDirectory())
    {
      return true;
    }

    String extension = getExtension(f);

    if (extension != null)
    {
      if (extension.equals("mp3"))
      {
        return true;
      }
      else
      {
        return false;
      }
    }
    return false;
  }

  /**
   * Return the file's extension.
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> File's extension has been returned.
   */
  private String getExtension(File file)
  {
    String ext = new String();
    String s = file.getName();
    int i = s.lastIndexOf('.');

    if (i>0 && (i<s.length()-1))
    {
      ext = s.substring(i+1).toLowerCase();
    }
    return ext;
  }


 /**
   * The description of this filter.
   * <p><b>pre:</b> True.
   * <p><b>post:</b> File's description has been returned.
   */
  public String getDescription()
  {
    return "mp3";
  }
} // eof.
