/*
 * MusicPlayer.java
 */

package catalog.guiutil;

import java.io.*;
import java.io.File;
import java.io.FileFilter;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.*;
import javax.media.ControllerListener;


/**
 * A class that create music player that plays mp3 files
 */
public class MusicPlayer extends JPanel
  implements ControllerListener
{
  // Instance variable for Player
  private Player player = null;

  // Instance variable for URL
  private URL mediaURL = null;

  // Instance variable for file
  private File file;

  // Instance variable for file chooser
  private JFileChooser fileChooser;

  /**
   * Constructs a <code>MusicPlayer</code> object.
   * <p><b>pre:</b> True.
   * <p><b>post:</b>An object has been created.
   */
  public MusicPlayer()
  {
    try
    {
      setLayout(new BorderLayout());
      openMusicFile();
      player = Manager.createPlayer(mediaURL);
      player.addControllerListener(this);
    }
    catch (Exception e)
    {
      System.err.println(e);
    }
    setBackground(Color.white);
  }

  /**
   * Play the music
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Music has been played .
   */
  public void start()
  {
    player.start();
  }

  /**
   * Stop the music
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Music has been stopped.
   */
  public void stop()
  {
    player.stop();
    player.deallocate();
  }

  /**
   * Close the music player
   * <p><b>pre:</b> Music player has been created.
   * <p><b>post:</b> Music has been destroyed.
   */
  public void destroy()
  {
    player.close();
  }


  /**
   * Open file chooser to select music file (.mp3)
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Music chooser dialog box has been created, user can
   *                 select music file (.mp3) from image chooser and music
   *                 will be played.
   */
	private void openMusicFile()
	{
    // get the current work directory
		File f = new File (System.getProperty("user.dir"));

    // set the current working directory to the Cover images Directly
  	String currentDirectory = f + file.separator + "music";

	  // Filechooser with the current working directory as the parameter
	  fileChooser = new JFileChooser(currentDirectory);

    // make visible only the mp3 files
	  fileChooser.setFileFilter(new MusicFilter());

    // can only select files no directories
  	fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

    // Set file to null if the file chooser is cancelled
		if(fileChooser.showOpenDialog(this)==JFileChooser.CANCEL_OPTION)
    {
			file = null;
      player.deallocate();
    }
		else
		{
			file = fileChooser.getSelectedFile();

			try
      {
			  mediaURL = file.toURL();
			}
      catch(MalformedURLException e)
      {
        System.err.println(e);
      }
		}
	}

  /**
   * Listen for ControllerEvent generated by component in this class
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> Player's visual component and control panel has been
   *                 added if the player is in Realized state
   */
  public synchronized void controllerUpdate(ControllerEvent event)
  {
    if (event instanceof RealizeCompleteEvent)
    {
      Component comp;
      if ((comp = player.getVisualComponent()) != null)
        add("Center", comp);
      if ((comp = player.getControlPanelComponent()) != null)
        add("South", comp);
      validate();
    }
  }
} // eof.
