/*
 * StartupGUI.java
 */

package catalog.guiutil;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * A class that renders the menu (4 buttons) and invoke other GUI
 * screens.
 */

public class StartupGUI extends JFrame
{
  // instance variables for buttons
  private JButton listButton,
                  searchButton,
                  addButton,
                  updateButton;

  // instance variable for panel
  private JPanel panel;

  // instance variable for color
  private Color bgColor,
                foreColor;

  // instance variable of ListGUI
  private ListGUI m_listGUI = null;



  public StartupGUI()
  {
    // set title for frame
    super("BOF Rock Classics");

    // set location for this frame
    setLocation(5,70);

    // create white color
    bgColor = new Color(255,255,255);
    
    // create navy color
    foreColor = new Color(50,70,220);

    // Listen for close window event
    addWindowListener(new WindowAdapter()
    {
       public void windowClosing(WindowEvent e)
       {
         StartupGUI.this.dispose();
         System.exit(0);
       }
    });

    // List Button
    listButton = new JButton(" List ");
    listButton.setForeground(foreColor);
    listButton.setMnemonic('L');
    listButton.setToolTipText("List all the CDs in the inventory");

    listButton.addActionListener
    (
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          m_listGUI = ListGUI.getInstance();
        }
      }
    );

    // Search Button
    searchButton = new JButton("Search");
    searchButton.setForeground(foreColor);
    searchButton.setMnemonic('S');
    searchButton.setToolTipText("Search for a CD");


    searchButton.addActionListener
    (
      new ActionListener()
      {
	      public void actionPerformed(ActionEvent e)
		    {
          new SearchGUI();
        }
      }
    );


    // Add Button
    addButton = new JButton("  Add ");
    addButton.setForeground(foreColor);
    addButton.setMnemonic('A');
    addButton.setToolTipText("Add a new CD");

    addButton.addActionListener
    (
      new ActionListener()
      {
	      public void actionPerformed(ActionEvent e)
		    {
          new AddGUI(StartupGUI.this);
        }
      }
    );


    // Update Button
    updateButton = new JButton("Update");
    updateButton.setForeground(foreColor);
    updateButton.setMnemonic('U');
    updateButton.setToolTipText("Update Stock for a CD");

    updateButton.addActionListener
    (
      new ActionListener()
      {
	      public void actionPerformed(ActionEvent e)
		    {
          new UpdateGUI(StartupGUI.this);
        }
      }
    );


    // Background panel for the startup window
    panel = new JPanel();
    panel.setBackground(bgColor);
    panel.setLayout(new GridLayout(1,4));

    // Add buttons to panel
    panel.add(listButton);
    panel.add(searchButton);
    panel.add(addButton);
    panel.add(updateButton);

    // Create spaces at both end
    JPanel fillPanel1 = new JPanel();
    JPanel fillPanel2 = new JPanel();
    JLabel space1 = new JLabel("    ");
    JLabel space2 = new JLabel("    ");

    fillPanel1.setBackground(bgColor);
    fillPanel2.setBackground(bgColor);

    fillPanel1.add(space1);
    fillPanel2.add(space2);

    // Add all components into the frame container
    Container Cp = getContentPane();
    Cp.setBackground(bgColor);
    Cp.add(panel, BorderLayout.CENTER);
    Cp.add(fillPanel1, BorderLayout.EAST);
    Cp.add(fillPanel2, BorderLayout.WEST);

    setSize(355,55);
    setVisible(true);
    validate();

    // Initialize DataStoreOptionGUI dialog
    new DataStoreOptionGUI(this);
  }


  /**
   * Returns a boolean indicating if ListGUI has been initialised
   * <p><b>pre:</b> True.
   * <p><b>post:</b> Return true if ListGUI has been initialiased else
   *                 return false.
   */
  private boolean isListGUIInstantiated()
  {
    if(m_listGUI!=null)
      return true;
    return false;
  }


  /**
   * Tell ListGUI (JTable) that there are changes in the internal data
   * stucture/datastore so that ListGUI will update its table view
   * <p><b>pre:</b> True.
   * <p><b>post:</b> update method in ListGUI will be invoked if ListGUI
   *                 has been initialiased.
   */
  public void propertyChanged()
  {
    if(isListGUIInstantiated())
      m_listGUI.update();
  }
} // eof.

