/*
 * UpdateGUI.java
 */

package catalog.guiutil;

import java.io.*;
import java.io.File;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.ImageIcon;
import catalog.datastore.*;
import catalog.compactdisk.*;
import catalog.CompactDiskCollection;


/**
 * A class that renders "Update CD stock" GUI (graphical user interface)
 * and handle events related to GUI components in this class.
 */

public class UpdateGUI extends JFrame implements ActionListener
{
  // Instance variables for labels
  private JLabel artistLabel,
                 titleLabel,
                 yearLabel,
                 stockLabel,
                 imageIconLabel;

  // Instance variables for panels
  private JPanel upperPanel,
                 middlePanel,
                 lowerPanel,
                 labelPanel,
                 fieldPanel,
                 innerLowerPanel1,
                 innerLowerPanel2;

  // Instance variables for text fields
  private JTextField artistTf,
                     titleTf,
                     yearTf,
                     stockTf;

  // Instance variables for button
  private JButton searchButton,
                  updateButton,
                  musicButton;

  // Instance variables for color
  private Color white,
                navyBlue,
                black,
                deepNavyBlue,
                gray;

  // Instance variable for music player
  MusicPlayer musicPlayer;

  // Instance variable for font
  private Font font;

  // Instance variable for image icon
  private Icon coverImage;

  // Instance variable for SearchGUI which instantiate this class
  private StartupGUI m_owner;

  private int fieldHeight = 26;
  private int fieldLenght = 200;


  /**
   * Constructs a <code>UpdateGUI</code> object using the given
   * arguments. GUI components has been rendered on screen
   * <p><b>pre:</b> Arguments are valid.
   * <p><b>post:</b>An object created and GUI components has been
   *                displayed on screen.
   */
  public UpdateGUI(StartupGUI owner)
  {
    super("Update stock for a CD");
    m_owner = owner;

    // set the dialog box to not resizable
    setResizable(false);

    // set location for this frame
    setLocation(568,80);

    // Create white color
    white = new Color(255,255,255);

    // Create navy blue color
    navyBlue = new Color(50,70,220);

    // Create black color
    deepNavyBlue = new Color(0,0,128);

    // Create black color
    black = new Color(0,0,0);

    // Create gray color
    gray = new Color(178,178,178);

    // Listen for close window event
    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        if(musicPlayer!=null)
        {
          musicPlayer.stop();
          musicPlayer.destroy();
        }
        UpdateGUI.this.dispose();
      }
    });


    // Search button
    searchButton = new JButton("Search");
    searchButton.setForeground(white);
    searchButton.setBackground(navyBlue);
    searchButton.setToolTipText("Search CD");
    searchButton.addActionListener(this);


    // Update button
    updateButton = new JButton(" Update");
    updateButton.setForeground(white);
    updateButton.setBackground(navyBlue);
    updateButton.setToolTipText("Upate CD record");
    updateButton.setEnabled(false);
    updateButton.addActionListener(this);



    // Play music button
    musicButton = new JButton("Play music");
    musicButton.setForeground(white);
    musicButton.setBackground(black);
    musicButton.setToolTipText("Play music");
    musicButton.addActionListener(this);


    // artist label
    artistLabel = new JLabel("Artist");
    artistLabel.setBackground(white);
    artistLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    artistLabel.setFont(font);


    // title label
    titleLabel = new JLabel("Title");
    titleLabel.setBackground(white);
    titleLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    titleLabel.setFont(font);


    // year label
    yearLabel = new JLabel("Year");
    yearLabel.setBackground(white);
    yearLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    yearLabel.setFont(font);


    // stock label
    stockLabel = new JLabel("Stock");
    stockLabel.setBackground(white);
    stockLabel.setForeground(deepNavyBlue);
    font = new Font ("Arial", Font.PLAIN, 13);
    stockLabel.setFont(font);


    // ImageIcon label
    imageIconLabel = new JLabel();
    imageIconLabel.setBackground(white);


    // artist textField
	  artistTf = new JTextField("",20);
	  artistTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    artistTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // title textField
	  titleTf = new JTextField("",20);
	  titleTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    titleTf.setAlignmentX(Component.LEFT_ALIGNMENT);


    // year textField
 	  yearTf = new JTextField("",20);
	  yearTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    yearTf.setAlignmentX(Component.LEFT_ALIGNMENT);
    yearTf.setEditable(false);


    // stock textField
	  stockTf = new JTextField("",20);
	  stockTf.setMinimumSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setPreferredSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setMaximumSize(new Dimension(fieldLenght,fieldHeight));
    stockTf.setAlignmentX(Component.LEFT_ALIGNMENT);
    stockTf.setEditable(false);


    // Create rigid area for text fields and labels

    // Label panel
   	labelPanel = new JPanel();
   	labelPanel.setOpaque(false);
    labelPanel.setBackground(white);
   	labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.Y_AXIS));
   	labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(artistLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(titleLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,10)));
    labelPanel.add(yearLabel);
    labelPanel.add(Box.createRigidArea(new Dimension(2,11)));
    labelPanel.add(stockLabel);

    // Field panel
    fieldPanel = new JPanel();
  	fieldPanel.setOpaque(false);
    fieldPanel.setBackground(white);
   	fieldPanel.setLayout(new BoxLayout(fieldPanel, BoxLayout.Y_AXIS));
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(artistTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(titleTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(yearTf);
   	fieldPanel.add(Box.createRigidArea(new Dimension(0,0)));
    fieldPanel.add(stockTf);


    // Upper panel for textfields and panels
    upperPanel = new JPanel();
    upperPanel.setBackground(white);
    upperPanel.setLayout(new GridLayout(1,2));
    upperPanel.add(labelPanel);
    upperPanel.add(fieldPanel);


    // Middle panel
    middlePanel = new JPanel();
    middlePanel.setBackground(white);
    middlePanel.add(imageIconLabel);


    // Lower panel
    lowerPanel = new JPanel();
    lowerPanel.setBackground(white);
    lowerPanel.setBorder(BorderFactory.createLineBorder(gray,2));
    lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.Y_AXIS));


    // Inner part of the lower panel for search button and clear button
    innerLowerPanel1 = new JPanel();
    innerLowerPanel1.setBackground(white);
  	innerLowerPanel1.setLayout(new BoxLayout(innerLowerPanel1, BoxLayout.X_AXIS));

    innerLowerPanel1.add(searchButton);
   	innerLowerPanel1.add(Box.createRigidArea(new Dimension(5,0)));
    innerLowerPanel1.add(updateButton);


    // Inner part of the lower panel for play music button
    innerLowerPanel2 = new JPanel();
    innerLowerPanel2.setBackground(white);
    innerLowerPanel2.setLayout(new BoxLayout(innerLowerPanel2, BoxLayout.X_AXIS));
    innerLowerPanel2.add(Box.createRigidArea(new Dimension(15,0)));
    innerLowerPanel2.add(musicButton);
    innerLowerPanel2.add(Box.createRigidArea(new Dimension(15,0)));


    // Add all panels into lower panel
    lowerPanel.add(Box.createRigidArea(new Dimension(0,20)));
    lowerPanel.add(innerLowerPanel1);
    lowerPanel.add(Box.createRigidArea(new Dimension(0,15)));
    lowerPanel.add(innerLowerPanel2);
    lowerPanel.add(Box.createRigidArea(new Dimension(0,0)));


    // Add all components into the frame container
    Container Cp = getContentPane();
    Cp.setBackground(white);
    Cp.setLayout(new GridLayout(3,1));
    Cp.add(upperPanel);
    Cp.add(middlePanel);
    Cp.add(lowerPanel);

    setSize(280,420);
    setVisible(true);
    validate();
  }

  /**
   * Listen for ActionEvent generated by component in this class
   * <p><b>pre:</b> Argument is valid.
   * <p><b>post:</b> Read user inputs and search CD if search button has
   *                 been clicked
   *                 CD data has been updated if the update button has
   *                 been clicked
   *                 Music player appears if music button has been clicked
   */
	public void actionPerformed(ActionEvent e)
	{
    if(e.getSource()==searchButton)
    {
      // validate input for artistTf
      if (artistTf.getText().length()==0)
      {
        JOptionPane.showMessageDialog(null, "Please enter artist name",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // validate input for titleTf
      if (titleTf.getText().length()==0)
      {
        JOptionPane.showMessageDialog(null, "Please enter title name",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      searchCD();
    }

    if(e.getSource()==updateButton)
    {
      // validate input for stockTf
      if ( (stockTf.getText().length()==0) ||
           (stockTf.getText().length()<0) ||
           (stockTf.getText().length()>5) )
      {
        JOptionPane.showMessageDialog(null, "Please enter valid stock value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // check whether stock is type of integer
      try
      {
        new Integer(stockTf.getText()).intValue();
      }
      catch(NumberFormatException err)
      {
        JOptionPane.showMessageDialog(null, "Please enter valid stock value",
                                      "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }

      // Invoke updateCD method in this class to update cd collection
      updateCD();

      // Tell StartupGUI that that there is a change in cd collection
      m_owner.propertyChanged();
    }

    if(e.getSource()==musicButton)
    {
      musicPlayer = new MusicPlayer();
      musicPlayer.start();
      lowerPanel.add(musicPlayer);
      lowerPanel.updateUI();
    }
	}


  /**
   * Read CD's artist name and title from GUI components and search for
   * the CD's data, and display the data on screen. Also enable the year
   * text field and stock text field so the user can update the stock count
   * of a particular CD
   * <p><b>pre:</b> Artist name and title has been input by the user.
   * <p><b>post:</b> CD details has been found and display on screen. yearTf
   *                 textfield and stockTf textfield has been enable for input
   */
  private void searchCD()
  {
    // Get input from text fields
    String artistInput = artistTf.getText();
    String titleInput = titleTf.getText();

    // Get instnace of CompactDiskCollection and get the requested cd
    CompactDiskCollection cds = CompactDiskCollection.getInstance();
    CompactDisk cd = cds.obtainCompactDisk(artistInput, titleInput);

    // Display warning dialog box if not item found
    if(cd==null)
    {
      JOptionPane.showMessageDialog(null, "CD not found!", "CD not found",
                                    JOptionPane.ERROR_MESSAGE);
      return;
    }

    // Display result on screen
    artistTf.setText(cd.getArtist());
    titleTf.setText(cd.getTitle());
    yearTf.setText(cd.getYear());
    stockTf.setText(new Integer(cd.getStockCount()).toString());

    // Enable the update button, year text field and stock text field
    updateButton.setEnabled(true);
    yearTf.setEditable(true);
    stockTf.setEditable(true);

    // Display cover image in the middle pabel
    coverImage = new ImageIcon(cd.getCover());
		imageIconLabel.setIcon(coverImage);
  }


  /**
   * Read input from GUI components and update the CD's data, and
   * upate data in internal data structure and datastore
   * <p><b>pre:</b> CD exist and has been found.
   * <p><b>post:</b> Data input has been read from GUI components.
   *                 CD detail in both the internal data structure
   *                 and datastore has been updated.
   */
  private void updateCD()
  {
    try
    {
      // Get input from text fields
      String artistInput = artistTf.getText();
      String titleInput = titleTf.getText();

      // Get instnace of CompactDiskCollection and get the requested cd
      CompactDiskCollection cds = CompactDiskCollection.getInstance();
      CompactDisk cd = cds.obtainCompactDisk(artistInput, titleInput);

      // Create an object of new compact disk
      CompactDisk newCD = new CompactDisk
      (
        cd.getProductCode(),
        cd.getArtist(),
        cd.getTitle(),
        cd.getYear(),
        cd.getCover(),
        new Integer(stockTf.getText()).intValue()
      );

      // Update the data in datastore (database/text file)
      DataStore data = DataStoreFactory.makeDataStore(
                       DataStoreOptionGUI.getDataStoreType());

      // Open datastore connection
      data.open();

      // Save data to datastore
      data.updateCompactDisk(newCD);

      // Update the stock count of particular cd
      cds.updateCompactDisk(newCD);
      System.out.println(cds.obtainCompactDisk(newCD.getArtist(), newCD.getTitle()).getStockCount());

      // Close datastore connection
      data.close();

    }
    catch(NumberFormatException err)
    {
      JOptionPane.showMessageDialog(null, "Invalid integer in stock field",
                                    "Error", JOptionPane.ERROR_MESSAGE);
    }
    catch(OpenDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch(SaveDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
    catch(CloseDataStoreException err)
    {
      JOptionPane.showMessageDialog(null, err.toString(), "Exception",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }
} // eof.
